//#########################################################################
// File: LCD.H
//
//#########################################################################
// Last change: 11.08.2010
//#########################################################################
// Compiler: AVR-GCC 4.1.1
//#########################################################################
// hk@holger-klabunde.de
//#########################################################################
//@{

#ifndef __LCD_H
#define __LCD_H

#include <avr/pgmspace.h>

// You should use only ONE of these !
#define HD44780
//#define KS073
//#define KS066		//not tested
//#define KS070		//not tested
// Todo: 1:1 sequences from datasheet for each device

#define LCD_LINES	2
#define LCD_ROWS	40
// Note: If you have a 1x16 display, you may have to use LCD_LINES 2 and LCD_ROWS 8.
// Most 1x16 displays are 2x8 displays organized in one line.

//#define LCD_BUSY_CHECK 	  // not implemented, RW has to be connected to ground
//#define LCD_CONTROLLERS	1 // not implemented, 2 for 4x40 LCD displays

//#define EADIP204     //EADIP204 Zeichensatzkonvertierungen
//#define CUSTOM_TABLE //Eigene Tabelle. In ConvertChar() selbst definieren
//Wenn beide nicht definiert werden wird eine Standardkonvertierung benutzt

 //Structure for LCD and 4 bit interface
 typedef struct sLCD{
  volatile uint8_t *rs_port;
  volatile uint8_t *rs_ddr;
  uint8_t rs_mask;
  volatile uint8_t *e_port;
  volatile uint8_t *e_ddr;
  uint8_t e_mask;
  volatile uint8_t *d4_port;
  volatile uint8_t *d4_ddr;
  uint8_t d4_mask;
  volatile uint8_t *d5_port;
  volatile uint8_t *d5_ddr;
  uint8_t d5_mask;
  volatile uint8_t *d6_port;
  volatile uint8_t *d6_ddr;
  uint8_t d6_mask;
  volatile uint8_t *d7_port;
  volatile uint8_t *d7_ddr;
  uint8_t d7_mask;
 }tLCD;

extern tLCD LCD1;
extern tLCD LCD2;
extern tLCD LCD3;

// Some usefull defines
#define NOP	 asm volatile ("nop" ::)
#define sbi(portn, bitn) ((portn)|=(1<<(bitn)))
#define cbi(portn, bitn) ((portn)&=~(1<<(bitn)))

//LCD1 pin definitions
 #define LCD1_RS_BIT	3 //LCD1 Register Select Pin
 #define LCD1_RS_PORT	PORTA
 #define LCD1_RS_DDR	DDRA

 #define LCD1_E_BIT		0 //LCD1 Chip Select Pin
 #define LCD1_E_PORT	PORTA
 #define LCD1_E_DDR	DDRA

 #define LCD1_D4_BIT	4
 #define LCD1_D4_DDR	DDRA
 #define LCD1_D4_PORT	PORTA

 #define LCD1_D5_BIT	5
 #define LCD1_D5_DDR	DDRA
 #define LCD1_D5_PORT	PORTA

 #define LCD1_D6_BIT	6
 #define LCD1_D6_DDR	DDRA
 #define LCD1_D6_PORT	PORTA

 #define LCD1_D7_BIT	7
 #define LCD1_D7_DDR	DDRA
 #define LCD1_D7_PORT	PORTA

//LCD2 pin definitions
 #define LCD2_RS_BIT	3 //LCD2 Register Select Pin
 #define LCD2_RS_PORT	PORTA
 #define LCD2_RS_DDR	DDRA

 #define LCD2_E_BIT		1 //LCD2 Chip Select Pin
 #define LCD2_E_PORT	PORTA
 #define LCD2_E_DDR	DDRA

 #define LCD2_D4_BIT	4
 #define LCD2_D4_DDR	DDRA
 #define LCD2_D4_PORT	PORTA

 #define LCD2_D5_BIT	5
 #define LCD2_D5_DDR	DDRA
 #define LCD2_D5_PORT	PORTA

 #define LCD2_D6_BIT	6
 #define LCD2_D6_DDR	DDRA
 #define LCD2_D6_PORT	PORTA

 #define LCD2_D7_BIT	7
 #define LCD2_D7_DDR	DDRA
 #define LCD2_D7_PORT	PORTA

//LCD3 pin definitions
 #define LCD3_RS_BIT	3 //LCD3 Register Select Pin
 #define LCD3_RS_PORT	PORTA
 #define LCD3_RS_DDR	DDRA

 #define LCD3_E_BIT		2 //LCD3 Chip Select Pin
 #define LCD3_E_PORT	PORTA
 #define LCD3_E_DDR	DDRA

 #define LCD3_D4_BIT	4
 #define LCD3_D4_DDR	DDRA
 #define LCD3_D4_PORT	PORTA

 #define LCD3_D5_BIT	5
 #define LCD3_D5_DDR	DDRA
 #define LCD3_D5_PORT	PORTA

 #define LCD3_D6_BIT	6
 #define LCD3_D6_DDR	DDRA
 #define LCD3_D6_PORT	PORTA

 #define LCD3_D7_BIT	7
 #define LCD3_D7_DDR	DDRA
 #define LCD3_D7_PORT	PORTA

void LCDInit(tLCD *lcd);
void LCDCls(tLCD *lcd);
void LCDPos(tLCD *lcd, unsigned char, unsigned char);
void LCDSetCGRAM(tLCD *lcd, unsigned char adr, unsigned char *buf);
void LCDWriteByte(tLCD *lcd, unsigned char);
void LCDWriteNibble(tLCD *lcd, unsigned char);
void LCDWrite(tLCD *lcd, char *);

void LCDWrite_p(tLCD *lcd, const char *s);
#define LCDWrite_P(a, text)   LCDWrite_p(a, PSTR(text))

char ConvertChar(char data);

void Delay1ms(unsigned int time);
void Delay1us(unsigned int time);

void LCDHexChar(tLCD *lcd, unsigned char by);
void LCDHexInt(tLCD *lcd, unsigned int by);
void LCDHexLong(tLCD *lcd, unsigned long l);

// Some security checks !
#if defined (HD44780) && defined (KS073)
 #error "Define HD44780 or KS073 only in lcd.h, NOT both !"
#endif

#if defined (HD44780) && defined (KS066)
 #error "Define HD44780 or KS066 only in lcd.h, NOT both !"
#endif

#if defined (KS066) && defined (KS073)
 #error "Define KS066 or KS073 only in lcd.h, NOT both !"
#endif

#if !defined (HD44780) && !defined (KS066) && !defined (KS070) && !defined (KS073)
 #error "Define at least HD44780 or KS066 or KS070 or KS073 in lcd.h !"
#endif

#endif
//@}
